///////////////////////////////////////////////////////////////////////////////
//
//    (C) Copyright 2011 EaseVault Technologies Inc.
//    All Rights Reserved
//
//    This software is part of a licensed software product and may
//    only be used or copied in accordance with the terms of that license.
//
///////////////////////////////////////////////////////////////////////////////

// CPlusPlusDemo.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "Tools.h"
#include "UnitTest.h"

#define	MAX_ERROR_MESSAGE_SIZE	1024

#define PrintMessage	wprintf //ToDebugger

VOID
__stdcall
DisconnectCallback();

BOOL
__stdcall
MessageCallback(
   IN		PMESSAGE_SEND_DATA pSendMessage,
   IN OUT	PMESSAGE_REPLY_DATA pReplyMessage);

void 
SendConfigInfoToFilter(ULONG FilterType,WCHAR* FilterFolder,ULONG IoRegistration ,ULONG AccessFlag);


VOID
Usage (
    VOID
    )
/*++

Routine Description

    Prints usage

Arguments

    None

Return Value

    None

--*/
{

	printf( "\nUsage:		CPlusPlusDemo  [command] <FilterFolder> <IoRegistration> <AccessFlag>\n" );
	printf( "\nCommands:\n" );
	printf( "		i ----- Install Driver\n" );
	printf( "		u ----- UnInstall Driver\n" );
	printf( "		t ----- Driver UnitTest\n" );
	printf( "		m ----- Start monitor filter,require FilterFolder,IoRgistration, if it is empty,it will use default value.\n" );
	printf( "		c ----- Start control filter,require FilterFolder,IoRgistration,AccessFlag if it is empty,it will use default value.\n" );
	printf( "\n		[FilterFolder]---- the folder mask which will be monitored. By default is c:\\filterTest\\*\n" );
	printf( "		[IoRegistration]---- the I/O requests register to filter. By default is all post I/O.\n" );
	printf( "		[AccessFlag]---- the I/O access flag. By default is ALLOW_MAX_RIGHT_ACCESS.\n" );
	
}


int _tmain(int argc, _TCHAR* argv[])
{
    DWORD	threadCount = 5;
	ULONG	filterType = FILE_SYSTEM_MONITOR;
	BOOL	ret = FALSE;
	
	//Purchase a license key with the link: http://www.easevault.com/Order.htm
    //Email us to request a trial key: info@easevault.com //free email is not accepted.

	char*	registerKey = "XXXXXXXX-XXXXXXXX-XXXXXXXX-XXXXXXXX-XXXXXXXX";

	if(argc <= 1)
	{
		Usage();
		return 1;
	}

	TCHAR op = *argv[1];

	switch(op)
	{
		case 'i': //install driver
		{
			//Install the driver only once.
			//After the installation, you can use command "fltmc unload EaseFlt" to unload the driver
			//or "fltmc load EaseFlt" to load the driver, or "fltmc" to check the status of the driver.
			ret = InstallDriver();	
			if( !ret )
			{
				PrintLastErrorMessage( L"InstallDriver failed.");
				return 1;
			}
			break;
		}

		case 'u': //uninstall driver
		{
			ret = UnInstallDriver();
			if( !ret )
			{
				PrintLastErrorMessage( L"UnInstallDriver failed.");
				return 1;
			}

			break;

		}

		case 't': //driver unit test 
		{
			ret = SetRegistrationKey(registerKey);
			if( !ret )
			{
				PrintLastErrorMessage( L"SetRegistrationKey failed.");
				return 1;
			}

			ret = RegisterMessageCallback(threadCount,MessageCallback,DisconnectCallback);

			if( !ret )
			{
				PrintLastErrorMessage( L"RegisterMessageCallback failed.");
				return 1;
			}

			//this the demo how to use the control filter.
			FilterDriverUnitTest();

			Disconnect();

			break;

		}

		case 'c': filterType = FILE_SYSTEM_CONTROL; //start control filter
		case 'm': ; //start monitor filter
		{

			TCHAR* filterFolder = GetFilterMask();
			ULONG ioRegistration = 0;
			ULONG accessFlag = ALLOW_MAX_RIGHT_ACCESS;			

			if( argc >= 3 )
			{
				filterFolder = argv[2];
			}
			
			if( argc >= 4 )
			{
				ioRegistration = _ttoi(argv[3]);
			}
			else
			{
				//Register the I/O request,which will be monitored or will be called back from filter.
				for (int i = 0; i < MAX_REQUEST_TYPE; i++ )
				{
					//register all post request
					if( (double)i/2 != i/2 )
					{
						ioRegistration |= 1<<i;
					}
				}     
			}

			if( argc >= 5 )
			{
				accessFlag = _ttoi(argv[4]);
			}


			_tprintf(_T("Start Monitor %s  ioregistration:0X%0X accessFlag:0X%0X\n\n Press any key to stop.\n"),filterFolder,ioRegistration,accessFlag);

			ret = SetRegistrationKey(registerKey);
			if( !ret )
			{
				PrintLastErrorMessage( L"SetRegistrationKey failed.");
				return 1;
			}

			ret = RegisterMessageCallback(threadCount,MessageCallback,DisconnectCallback);

			if( !ret )
			{
				PrintLastErrorMessage( L"RegisterMessageCallback failed.");
				return 1;
			}

			//this the demo how to use the control filter.
			SendConfigInfoToFilter(filterType,filterFolder,ioRegistration,accessFlag);

			system("pause");

			Disconnect();

			break;

		}
		default:
			{
				Usage(); 
				return 1;
			}


	}

		
	return 0;
}




VOID
PrintMessageInfo( IN	PMESSAGE_SEND_DATA pSendMessage )
{
	WCHAR userName[MAX_PATH];
	WCHAR domainName[MAX_PATH];

	int userNameSize = MAX_PATH;
	int domainNameSize = MAX_PATH;
	SID_NAME_USE snu;

	__try
	{
		BOOL ret = LookupAccountSid( NULL,
									pSendMessage->Sid,
									userName,
									(LPDWORD)&userNameSize,
									domainName,
									(LPDWORD)&domainNameSize,
									&snu); 
	
		if( pSendMessage->Status > STATUS_ERROR )
		{
			ChangeColour(FOREGROUND_RED);
		}
		else if ( pSendMessage->Status > STATUS_WARNING )
		{
			ChangeColour(FOREGROUND_RED|FOREGROUND_GREEN);
		}

		PrintMessage( L"\nId# %d MessageType:0X%0x UserName:%ws\\%ws\nProcessId:%d ThreadId:%d Return Status:%0x\nFileSize:%I64d Attributes:%0x FileName:%ws\n"
			,pSendMessage->MessageId,pSendMessage->MessageType,domainName,userName
			,pSendMessage->ProcessId,pSendMessage->ThreadId,pSendMessage->Status
			,pSendMessage->FileSize,pSendMessage->FileAttributes,pSendMessage->FileName);

		ChangeColour(FOREGROUND_RED|FOREGROUND_GREEN|FOREGROUND_BLUE);
	
	  
		switch( pSendMessage->MessageType )
		{

			case PRE_CREATE:
			case POST_CREATE:
			{
				//for Disposition,ShareAccess,DesiredAccess,CreateOptions Please reference Winddows API CreateFile
				//http://msdn.microsoft.com/en-us/library/aa363858%28v=vs.85%29.aspx

			   PrintMessage( L"CreateRequest DesiredAccess=%d Disposition=%d ShareAccess=%d CreateOptions=0x%0x CreateStatus = %d \n"
					,pSendMessage->DesiredAccess,pSendMessage->Disposition,pSendMessage->ShareAccess,pSendMessage->CreateOptions,pSendMessage->CreateStatus);
			   
			   break;
  			}

			case PRE_QUERY_INFORMATION:
			case POST_QUERY_INFORMATION:
			case PRE_SET_INFORMATION:
			case POST_SET_INFORMATION:
			{
				 //FltQueryInformationFile API,http://msdn.microsoft.com/en-us/library/windows/hardware/ff543439%28v=vs.85%29.aspx
				 //FltSetInformationFile API,http://msdn.microsoft.com/en-us/library/windows/hardware/ff544516%28v=VS.85%29.aspx
				 PrintMessage( L"Query/Set information request FileInformationClass = %d \n",pSendMessage->InfoClass);

				 //for POST_QUERY_INFORMATION request, the pSendMessage->DataBuffer contains the data which return from the file system.
				 //for PRE_SET_INFORMATION request, the pSendMessage->DataBuffer contains the data which will write down to the file system.
				 
				 break;
			}

			case PRE_QUERY_SECURITY:
			case POST_QUERY_SECURITY:
			case PRE_SET_SECURITY:
			case POST_SET_SECURITY:
			{
				 //FltQuerySecurityObject API,http://msdn.microsoft.com/en-us/library/windows/hardware/ff543441%28v=vs.85%29.aspx
				 //FltSetSecurityObject API,http://msdn.microsoft.com/en-us/library/ff544538
				 PrintMessage( L"Query/Set information request SecurityInformation  = %d \n",pSendMessage->InfoClass);

				 //for POST_QUERY_SECURITY request, the pSendMessage->DataBuffer contains the data which return from the file system.
				 //for PRE_SET_SECURITY request, the pSendMessage->DataBuffer contains the data which will write down to the file system.

				 break;
			}


			case PRE_DIRECTORY:
			case POST_DIRECTORY:
			{
				//FltQueryDirectoryFile API,http://msdn.microsoft.com/en-us/library/windows/hardware/ff543433%28v=vs.85%29.aspx
				PrintMessage( L"Browse directory request, DataBuffer:%0x FileInformationClass  = %d \n"	,pSendMessage->DataBuffer,pSendMessage->InfoClass);

				//for POST_DIRECTORY request, the pSendMessage->DataBuffer contains the data which return from the file system.

				 break;
			}

			case PRE_FASTIO_READ:
			case POST_FASTIO_READ:
			case PRE_CACHE_READ:
			case POST_CACHE_READ: 
			case PRE_NOCACHE_READ:
			case POST_NOCACHE_READ:
			case PRE_PAGING_IO_READ:
			case POST_PAGING_IO_READ:
			{
				 //FltReadFile API,http://msdn.microsoft.com/en-us/library/windows/hardware/ff544286%28v=vs.85%29.aspx
				PrintMessage( L"Id#:%d Read requst, Offset = %I64d Length = %d returnLength = %d  \n"
				   ,pSendMessage->MessageId,pSendMessage->Offset,pSendMessage->Length,pSendMessage->DataBufferLength);
			   
			   //display the read data return from file system.
			   //printf("data:%s",pSendMessage->DataBuffer);    //it is ansi code characters
			   //wprintf("data:%ws",pSendMessage->DataBuffer);	//it is unicode characters

				//for post read request, the pSendMessage->DataBuffer contains the data which return from the file system.

			   break;
			}

			case PRE_FASTIO_WRITE:
			case POST_FASTIO_WRITE:
			case PRE_CACHE_WRITE:
			case POST_CACHE_WRITE: 
			case PRE_NOCACHE_WRITE:
			case POST_NOCACHE_WRITE:
			case PRE_PAGING_IO_WRITE:
			case POST_PAGING_IO_WRITE:
			{
				 //FltWriteFile API,http://msdn.microsoft.com/en-us/library/windows/hardware/ff544610%28v=vs.85%29.aspx
				PrintMessage( L"WRITE requst, Offset = %I64d Length = %d returnLength = %d \n"
				   ,pSendMessage->Offset,pSendMessage->Length,pSendMessage->DataBufferLength);
				
			   //display the write data to file system.
			   //printf("data:%s",pSendMessage->DataBuffer);    //it is ansi code characters
			   //wprintf("data:%ws",pSendMessage->DataBuffer);	//it is unicode characters

			   //for pre write request, the pSendMessage->DataBuffer contains the data which will write down to the file system.
				
				break;
			}	

			default: break;
		}

	}
	__except( EXCEPTION_EXECUTE_HANDLER  )
	{
		PrintErrorMessage( L"PrintMessageInfo failed.",GetLastError());     
	}

return ;

}


BOOL 
ProcessControlFilter( IN	PMESSAGE_SEND_DATA pSendMessage,IN OUT	PMESSAGE_REPLY_DATA pReplyMessage )
{
	BOOL ret = TRUE;


	try
	{
		
		switch( pSendMessage->MessageType )
		{
		
		case PRE_CREATE:
			{
				//
				//For reparse file/folder open.
				//
				if( IsTestFile(pSendMessage->FileName,pSendMessage->FileNameLength) )
				{
				   PrintMessage(L"File:%ws will be redirected to new file:%ws open",pSendMessage->FileName,GetTestReparseFileName());

				   //the file name must be unicode string, file name length must be less than MAX_PATH.
				   memcpy(pReplyMessage->DataBuffer,GetTestReparseFileName(),(ULONG)wcslen(GetTestReparseFileName())*sizeof(WCHAR));
				   pReplyMessage->DataBufferLength =(ULONG)wcslen(GetTestReparseFileName())*sizeof(WCHAR);
				   
				   pReplyMessage->FilterStatus = FILTER_MESSAGE_IS_DIRTY|FILTER_COMPLETE_PRE_OPERATION|FILTER_DATA_BUFFER_IS_UPDATED;
				   pReplyMessage->ReturnStatus = STATUS_REPARSE;
				}

				break;
			}

		case PRE_QUERY_INFORMATION:
			{
				
				//
				//Return your own information without go down to the file system.
				//
				
				if( IsTestFile(pSendMessage->FileName,pSendMessage->FileNameLength) )
				{
					FILE_BASIC_INFORMATION basicInfo;

					if( pSendMessage->InfoClass == FileBasicInformation )
					{
						basicInfo.ChangeTime = basicInfo.LastWriteTime 
						= basicInfo.CreationTime = basicInfo.LastAccessTime = GetTestFileTime();

						basicInfo.FileAttributes = FILE_ATTRIBUTE_READONLY;

						memcpy(pReplyMessage->DataBuffer,&basicInfo,sizeof(FILE_BASIC_INFORMATION));
						pReplyMessage->DataBufferLength = sizeof(FILE_BASIC_INFORMATION);

						pReplyMessage->FilterStatus = FILTER_MESSAGE_IS_DIRTY|FILTER_COMPLETE_PRE_OPERATION|FILTER_DATA_BUFFER_IS_UPDATED;
						pReplyMessage->ReturnStatus = STATUS_SUCCESS;
					}


					//for other information class is the same process.
				}			

				break;

			}
			
		case POST_QUERY_INFORMATION:
			{
				
				//
				//Change the information which returns from the file system.
				//
				
				if( IsTestFile(pSendMessage->FileName,pSendMessage->FileNameLength) )
				{				
					if( pSendMessage->InfoClass == FileBasicInformation )
					{
						PFILE_BASIC_INFORMATION basicInfo = (PFILE_BASIC_INFORMATION)pSendMessage->DataBuffer;

						basicInfo->ChangeTime = basicInfo->LastWriteTime = GetTestFileTime();
						basicInfo->FileAttributes |= FILE_ATTRIBUTE_READONLY;

						memcpy(pReplyMessage->DataBuffer,basicInfo,sizeof(FILE_BASIC_INFORMATION));
						pReplyMessage->DataBufferLength = sizeof(FILE_BASIC_INFORMATION);

						pReplyMessage->FilterStatus = FILTER_MESSAGE_IS_DIRTY|FILTER_DATA_BUFFER_IS_UPDATED;
						pReplyMessage->ReturnStatus = STATUS_SUCCESS;
					}

					//for other information class is the same process.
				}
			

				break;
			}


		case PRE_SET_INFORMATION:
			{
				

				//
				//Change the information before write down to the file system.
				//
				
				if( IsTestFile(pSendMessage->FileName,pSendMessage->FileNameLength) )
				{
					
					if( pSendMessage->InfoClass == FileBasicInformation )
					{
						PFILE_BASIC_INFORMATION basicInfo = (PFILE_BASIC_INFORMATION)pSendMessage->DataBuffer;

						basicInfo->ChangeTime = basicInfo->LastWriteTime = GetTestFileTime();
						basicInfo->FileAttributes = FILE_ATTRIBUTE_READONLY;

						memcpy(pReplyMessage->DataBuffer,basicInfo,sizeof(FILE_BASIC_INFORMATION));
						pReplyMessage->DataBufferLength = sizeof(FILE_BASIC_INFORMATION);

						pReplyMessage->FilterStatus = FILTER_MESSAGE_IS_DIRTY|FILTER_DATA_BUFFER_IS_UPDATED;
						pReplyMessage->ReturnStatus = STATUS_SUCCESS;

					}

					//for other information class is the same process.
				}

			
				break;
			}

		//For POST_SET_INFORMATION, you can't change anything, since the data was written to the file system.
	   
		//
		//For PRE_DIRECTORY,you can complete it and return your own directory data,the data structure format
		//must be the same as information class from the request,you can reference POST_DIRECTORY.

		case POST_DIRECTORY:
			{
				//
				//Change the information which returns from the file system.
				//
				if(STATUS_SUCCESS == pSendMessage->Status && IsTestFolder(pSendMessage->FileName) )
				{					


					//here we use FileBothDirectoryInformation as example, for other classes, you can do the same operation.
					if( pSendMessage->InfoClass == FileBothDirectoryInformation )
					{
						memcpy(pReplyMessage->DataBuffer,pSendMessage->DataBuffer,pSendMessage->DataBufferLength);
						pReplyMessage->DataBufferLength = pSendMessage->DataBufferLength;
						pReplyMessage->ReturnStatus = STATUS_SUCCESS;

						PFILE_BOTH_DIR_INFORMATION		fileBothDirInfo = (PFILE_BOTH_DIR_INFORMATION)pReplyMessage->DataBuffer;
						ULONG	returnBufferLength = 0;
						ULONG	totalBufferLength = pReplyMessage->DataBufferLength;

						if( NULL == fileBothDirInfo )
						{
							PrintErrorMessage(L"POST_DIRECTORY return empty databuffer",0);
						}

						BOOL isDatabufferUpdated = FALSE;

						while( fileBothDirInfo )
						{
							ULONG nextOffset = fileBothDirInfo->NextEntryOffset;

							PrintMessage(L"FileName:%ws Attributes:0x%0x FileSize:%I64d\n"
								,fileBothDirInfo->FileName,fileBothDirInfo->FileAttributes,fileBothDirInfo->EndOfFile.QuadPart);

							//You can modify the data of the fileBothDirInfo, but must be careful to change the size of the entry.
							//This structure must be aligned on a LONGLONG (8-byte) boundary. If a buffer contains two or more of these structures, 
							//the NextEntryOffset value in each entry, except the last, falls on an 8-byte boundary. 

							//Test: Hide all the files whose file size greater than 1024 bytes.
							if( fileBothDirInfo->EndOfFile.QuadPart > 1024 )
							{				
								if( nextOffset != 0 )
								{
									PrintMessage(L"File %ws is filtered",fileBothDirInfo->FileName);
									memmove (fileBothDirInfo,(PUCHAR)fileBothDirInfo + nextOffset,totalBufferLength - returnBufferLength - nextOffset );
									isDatabufferUpdated = TRUE;
									continue;
								}
								else
								{
									fileBothDirInfo->NextEntryOffset = 0;
								}
							}

							//Test: Add ReadOnly to the FileAttributes, change FileSize to test file size for all files;
							if(NULL == (fileBothDirInfo->FileAttributes & FILE_ATTRIBUTE_DIRECTORY) )
							{
								PrintMessage(L"File %ws attributs:0x%0x was added ReadOnly,Filesize:%I64d was changed to 1234567890"
									,fileBothDirInfo->FileName,fileBothDirInfo->FileAttributes,fileBothDirInfo->EndOfFile.QuadPart);

								fileBothDirInfo->FileAttributes |= FILE_ATTRIBUTE_READONLY;
								fileBothDirInfo->EndOfFile.QuadPart = GetTestFileSize();
								isDatabufferUpdated = TRUE;
							}

							returnBufferLength += nextOffset;
							
							if( nextOffset == 0)
							{
								returnBufferLength += sizeof(FILE_BOTH_DIR_INFORMATION) + fileBothDirInfo->FileNameLength - sizeof(WCHAR);
								fileBothDirInfo = NULL;
							}
							else
							{
								fileBothDirInfo = (PFILE_BOTH_DIR_INFORMATION)((PUCHAR)fileBothDirInfo + nextOffset);
							}

						}

						pReplyMessage->DataBufferLength = returnBufferLength;

						//if you filter out all the files, you can return no more files status. The system won't generate another request.
						if( returnBufferLength == 0 )
						{
							pReplyMessage->ReturnStatus = STATUS_NO_MORE_FILES;	
						}

						if( isDatabufferUpdated )
						{
							PrintMessage(L"Update directory buffer,original length:%d new length:%d",pSendMessage->DataBufferLength,pReplyMessage->DataBufferLength);
							pReplyMessage->FilterStatus = FILTER_MESSAGE_IS_DIRTY|FILTER_DATA_BUFFER_IS_UPDATED;
						}
					}
				}

				break;
			}

			case PRE_FASTIO_READ:
			case PRE_CACHE_READ:
			case PRE_NOCACHE_READ:
			case PRE_PAGING_IO_READ:
			{
				//
				//Return your own data without go down to the file system.
				//
				if( IsTestFile(pSendMessage->FileName,pSendMessage->FileNameLength) )
				{
					ULONG dataLength = (ULONG)strlen(GetReplaceData());

					if( pSendMessage->Length < dataLength )
					{
						dataLength = pSendMessage->Length;
					}

					memcpy(pReplyMessage->DataBuffer,GetReplaceData(),dataLength);
					pReplyMessage->DataBufferLength = dataLength;

					pReplyMessage->FilterStatus = FILTER_MESSAGE_IS_DIRTY|FILTER_COMPLETE_PRE_OPERATION|FILTER_DATA_BUFFER_IS_UPDATED;
					pReplyMessage->ReturnStatus = STATUS_SUCCESS;
				}

			   break;
			}

			case POST_FASTIO_READ:
			case POST_CACHE_READ: 
			case POST_NOCACHE_READ:
			case POST_PAGING_IO_READ:
			{
				//
				//Modify the data which returns from the file system.
				//
				
				if( IsTestFile(pSendMessage->FileName,pSendMessage->FileNameLength) )
				{
					
					memcpy(pReplyMessage->DataBuffer,pSendMessage->DataBuffer,pSendMessage->DataBufferLength);
					//merge test data to the beginning of the buffer.
					ULONG dataLength = (ULONG)strlen(GetReplaceData());

					if( pSendMessage->Length < dataLength )
					{
						dataLength = pSendMessage->Length;
					}

					memcpy(pReplyMessage->DataBuffer,GetReplaceData(),dataLength);
					pReplyMessage->DataBufferLength = dataLength;

									
					pReplyMessage->FilterStatus = FILTER_MESSAGE_IS_DIRTY|FILTER_DATA_BUFFER_IS_UPDATED;
					pReplyMessage->ReturnStatus = STATUS_SUCCESS;
				}


			   break;
			}


			case PRE_FASTIO_WRITE:
			case PRE_CACHE_WRITE:
			case PRE_NOCACHE_WRITE:
			case PRE_PAGING_IO_WRITE:
			{
				//
				//Modiy the write data buffer before it goes down to the file system.
				//
				
				if( IsTestFile(pSendMessage->FileName,pSendMessage->FileNameLength) )
				{
					memcpy(pReplyMessage->DataBuffer,pSendMessage->DataBuffer,pSendMessage->DataBufferLength);
					//merge test data to the beginning of the buffer.
					ULONG dataLength = (ULONG)strlen(GetReplaceData());

					if( pSendMessage->Length < dataLength )
					{
						dataLength = pSendMessage->Length;
					}

					memcpy(pReplyMessage->DataBuffer,GetReplaceData(),dataLength);
					pReplyMessage->DataBufferLength = dataLength;


					pReplyMessage->FilterStatus = FILTER_MESSAGE_IS_DIRTY|FILTER_DATA_BUFFER_IS_UPDATED;
					pReplyMessage->ReturnStatus = STATUS_SUCCESS;
				}

			   break;
			}

			//For post write operation, the data was written to the file system,at this point, we can't change anything.

			default: break;
		
		}
	}
	catch(...)
	{
		PrintErrorMessage( L"ProcessControlFilter exception.",GetLastError());     
		ret = FALSE;
	}

	return ret;
}

BOOL
__stdcall
MessageCallback(
   IN		PMESSAGE_SEND_DATA pSendMessage,
   IN OUT	PMESSAGE_REPLY_DATA pReplyMessage)
{

	BOOL	ret = TRUE;

	PrintMessageInfo(pSendMessage);

	if(pReplyMessage)
	{
		ret = ProcessControlFilter(pSendMessage,pReplyMessage);
	}

	return ret;
}

VOID
__stdcall
DisconnectCallback()
{
	PrintLastErrorMessage(L"Filter connection was disconnected.");
	return;
}


void 
SendConfigInfoToFilter(ULONG FilterType,WCHAR* FilterFolder,ULONG IoRegistration ,ULONG AccessFlag)
{
   //Reset all filter confing setting.
    ResetConfigData();

	//Set the filter maximum wait time for response from the user mode call back function.
	SetConnectionTimeout(30);       

	//Set the filter type to file system monitor/call back filter.
    SetFilterType(FilterType);

	//Setup the filter rule,filter mask is the folder to be monitored,
	//accessFlag is the I/O access control of the folder, only affect callback filter,
	//reparseMask is the destination folder for reparse file open, only when the accessFlag = REPARSE_FILE_OPEN.

	ULONG accessFlag = AccessFlag;
	
	if( FilterFolder )
	{
		AddFilterRule(accessFlag,FilterFolder,L"");
	}


	accessFlag = REPARSE_FILE_OPEN;
	WCHAR* filterMask2 = L"c:\\filterTest\\reparsetest\\*"; 
	WCHAR* reparseMask = L"c:\\reparse\\*";
	
    AddFilterRule(accessFlag,filterMask2,reparseMask);

	//Exclude the process Id from the filter.
    AddExcludedProcessId(GetCurrentProcessId());
	
	RegisterIoRequest(IoRegistration);
}


