﻿///////////////////////////////////////////////////////////////////////////////
//
//    (C) Copyright 2011 EaseVault Technologies Inc.
//    All Rights Reserved
//
//    This software is part of a licensed software product and may
//    only be used or copied in accordance with the terms of that license.
//
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace CSharpDemo
{
    public partial class FilterDemo : Form
    {
        //Purchase a license key with the link: http://www.easevault.com/Order.htm
        //Email us to request a trial key: info@easevault.com //free email is not accepted.
        string registerKey = "XXXXXXXX-XXXXXXXX-XXXXXXXX-XXXXXXXX-XXXXXXXX";

        public FilterDemo()
        {
            InitializeComponent();
        }

    
        private void button_Start_Click(object sender, EventArgs e)
        {
            bool ret = EaseFilter.StartFilter(  registerKey
                                                ,(int)configSetting.FilterConnectionThreads
                                                , new EaseFilter.FilterDelegate(FilterCallback)
                                                , new EaseFilter.DisconnectDelegate(DisconnectCallback));
            if (!ret)
            {
                MessageBox.Show("Start filter failed." + EaseFilter.GetLastErrorMessage());
                return;
            }

            button_Start.Enabled = false;
            button_Stop.Enabled = true;

            SendConfigInfoToFilter();
        }

        private void button_Stop_Click(object sender, EventArgs e)
        {
            EaseFilter.StopFilter();
            button_Start.Enabled = true;
            button_Stop.Enabled = false;
        }

        private void button_Clear_Click(object sender, EventArgs e)
        {
            messageInfo.InitListView();
        }

        private void button_InstallDriver_Click(object sender, EventArgs e)
        {
            if (!EaseFilter.InstallDriver())
            {
                MessageBox.Show(EaseFilter.GetLastErrorMessage(), "Install Driver Failed", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                button_InstallDriver.Enabled = false;
                button_UnInstallDriver.Enabled = true;
            }
        }

        private void button_UnInstallDriver_Click(object sender, EventArgs e)
        {
            if (!EaseFilter.UnInstallDriver())
            {
                MessageBox.Show(EaseFilter.GetLastErrorMessage(), "UnInstall Driver Failed", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                button_InstallDriver.Enabled = true;
                button_UnInstallDriver.Enabled = false;
            }
        }

        Boolean FilterCallback(IntPtr sendDataPtr, IntPtr replyDataPtr)
        {
            Boolean ret = true;

            try
            {
                EaseFilter.MessageSendData messageSend = new EaseFilter.MessageSendData();
                messageSend = (EaseFilter.MessageSendData)Marshal.PtrToStructure(sendDataPtr, typeof(EaseFilter.MessageSendData));

                if (EaseFilter.MESSAGE_SEND_VERIFICATION_NUMBER != messageSend.VerificationNumber)
                {
                    MessageBox.Show("Received message corrupted.Please check if the MessageSendData structure is correct.");
                    return false;
                }

                messageInfo.AddMessage(messageSend);

                if (configSetting.FilterType == EaseFilter.FilterType.FILE_SYSTEM_CONTROL
                    && replyDataPtr.ToInt64() != 0
                    && configSetting.EnableFilterTest)
                {
                    ret = filterTest.ProcessReplyMessage(messageSend, replyDataPtr);
                }

                return ret;
            }
            catch (Exception ex)
            {
                EventMessage.LogErrorMessage("filter callback exception." + ex.Message);
                return false;
            }

        }

        void DisconnectCallback()
        {
            MessageBox.Show("Filter Disconnected." + EaseFilter.GetLastErrorMessage(), "Filter Disconnected.", MessageBoxButtons.OK, MessageBoxIcon.Warning);
        }


    }
}
